from .models import Thread


def user_messages(request):
    c = {}
    if request.user.is_authenticated():
        c["inbox_threads"] = Thread.inbox(request.user)
        c["unread_threads"] = Thread.unread(request.user)
    return c



from django import forms

from django.contrib.auth import get_user_model

from .hooks import hookset
from .models import Message


class UserModelChoiceField(forms.ModelChoiceField):

    def label_from_instance(self, obj):
        return hookset.display_name(obj)


class UserModelMultipleChoiceField(forms.ModelMultipleChoiceField):

    def label_from_instance(self, obj):
        return hookset.display_name(obj)


class NewMessageForm(forms.Form):

    subject = forms.CharField()
    to_user = UserModelChoiceField(queryset=get_user_model().objects.none)
    content = forms.CharField(widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super(NewMessageForm, self).__init__(*args, **kwargs)
        self.fields["to_user"].queryset = hookset.get_user_choices(self.user)
        if self.initial.get("to_user") is not None:
            qs = self.fields["to_user"].queryset.filter(pk=self.initial["to_user"])
            self.fields["to_user"].queryset = qs

    def save(self):
        data = self.cleaned_data
        return Message.new_message(
            self.user, [data["to_user"]], data["subject"], data["content"]
        )


class NewMessageFormMultiple(forms.Form):

    subject = forms.CharField()
    to_user = UserModelMultipleChoiceField(get_user_model().objects.none)
    content = forms.CharField(widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super(NewMessageFormMultiple, self).__init__(*args, **kwargs)
        self.fields["to_user"].queryset = hookset.get_user_choices(self.user)
        if self.initial.get("to_user") is not None:
            qs = self.fields["to_user"].queryset.filter(pk__in=self.initial["to_user"])
            self.fields["to_user"].queryset = qs

    def save(self):
        data = self.cleaned_data
        return Message.new_message(
            self.user, data["to_user"], data["subject"], data["content"]
        )


class MessageReplyForm(forms.Form):

    content = forms.CharField(widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        self.thread = kwargs.pop("thread")
        self.user = kwargs.pop("user")
        super(MessageReplyForm, self).__init__(*args, **kwargs)

    def save(self):
        return Message.new_reply(
            self.thread, self.user, self.cleaned_data["content"]
        )



from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.views.generic import TemplateView, DetailView
from django.views.generic.edit import FormMixin
from django.views.decorators.http import require_POST

from account.decorators import login_required
from account.mixins import LoginRequiredMixin

from .forms import MessageReplyForm, NewMessageForm, NewMessageFormMultiple
from .models import Thread


class InboxView(LoginRequiredMixin, TemplateView):
    template_name = "pinax/messages/inbox.html"

    def get_context_data(self, **kwargs):
        context = super(InboxView, self).get_context_data(**kwargs)
        context.update({
            "threads": Thread.ordered(Thread.inbox(self.request.user)),
            "threads_unread": Thread.ordered(Thread.unread(self.request.user))
        })
        return context


class ThreadView(LoginRequiredMixin, FormMixin, DetailView):
    model = Thread
    context_object_name = "thread"
    template_name = "pinax/messages/thread_detail.html"
    form_class = MessageReplyForm

    def get_queryset(self):
        qs = super(ThreadView, self).get_queryset()
        qs = qs.filter(userthread__user=self.request.user).distinct()
        return qs

    def get_form_kwargs(self):
        kwargs = super(ThreadView, self).get_form_kwargs()
        kwargs.update({
            "user": self.request.user,
            "thread": self.object
        })
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(ThreadView, self).get_context_data(**kwargs)
        context.update({
            "form": self.get_form()
        })
        return context

    def get_success_url(self):
        return reverse("messages_inbox")

    def get(self, request, *args, **kwargs):
        response = super(ThreadView, self).get(request, *args, **kwargs)
        self.object.userthread_set.filter(user=request.user).update(unread=False)
        return response

    def form_valid(self, form):
        form.save()
        return super(ThreadView, self).form_valid(form)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)


@login_required
def message_create(
    request, user_id=None,
    template_name="user_messages/message_create.html",
    form_class=None, multiple=False
):
    if form_class is None:
        if multiple:
            form_class = NewMessageFormMultiple
        else:
            form_class = NewMessageForm

    if user_id is not None:
        user_id = [int(user_id)]
    elif "to_user" in request.GET and request.GET["to_user"].isdigit():
        user_id = map(int, request.GET.getlist("to_user"))
    if not multiple and user_id:
        user_id = user_id[0]
    initial = {"to_user": user_id}
    if request.method == "POST":
        form = form_class(request.POST, user=request.user, initial=initial)
        if form.is_valid():
            msg = form.save()
            return HttpResponseRedirect(msg.get_absolute_url())
    else:
        form = form_class(user=request.user, initial=initial)
    return render_to_response(template_name, {
        "form": form
    }, context_instance=RequestContext(request))


@login_required
@require_POST
def thread_delete(request, thread_id):
    qs = Thread.objects.filter(userthread__user=request.user)
    thread = get_object_or_404(qs, pk=thread_id)
    thread.userthread_set.filter(user=request.user).update(deleted=True)
    return HttpResponseRedirect(reverse("messages_inbox"))



from django.conf.urls import patterns, url

from . import views

urlpatterns = patterns(
    "",
    url(r"^inbox/$", views.InboxView.as_view(), name="messages_inbox"),
    url(r"^create/$", views.message_create, name="message_create"),
    url(r"^create/(?P<user_id>\d+)/$", views.message_create, name="message_create"),
    url(r"^thread/(?P<thread_id>\d+)/$", views.ThreadView.as_view(), name="messages_thread_detail"),
    url(r"^thread/(?P<thread_id>\d+)/delete/$", views.thread_delete, name="messages_thread_delete"),
)



import os

from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.test import TestCase as BaseTestCase

from django.contrib.auth import get_user_model

from ..models import Thread, Message


class login(object):

    def __init__(self, testcase, user, password):
        self.testcase = testcase
        success = testcase.client.login(username=user, password=password)
        self.testcase.assertTrue(
            success,
            "login with username=%r, password=%r failed" % (user, password)
        )

    def __enter__(self):
        pass

    def __exit__(self, *args):
        self.testcase.client.logout()


class TestCaseMixin(object):
    def get(self, url_name, *args, **kwargs):
        data = kwargs.pop("data", {})
        return self.client.get(reverse(url_name, args=args, kwargs=kwargs), data)

    def post(self, url_name, *args, **kwargs):
        data = kwargs.pop("data", {})
        return self.client.post(reverse(url_name, args=args, kwargs=kwargs), data)

    def login(self, user, password):
        return login(self, user, password)

    def reload(self, obj):
        return obj.__class__._default_manager.get(pk=obj.pk)

    def assert_renders(self, tmpl, context, value):
        tmpl = Template(tmpl)
        self.assertEqual(tmpl.render(context), value)


class TestCase(BaseTestCase, TestCaseMixin):
    pass


class BaseTest(TestCase):
    def setUp(self):
        User = get_user_model()
        self.brosner = User.objects.create_superuser(
            "brosner", "brosner@brosner.brosner", "abc123")
        self.jtauber = User.objects.create_superuser(
            "jtauber", "jtauber@jtauber.jtauber", "abc123")
        if hasattr(self, "template_dirs"):
            self._old_template_dirs = settings.TEMPLATE_DIRS
            settings.TEMPLATE_DIRS = self.template_dirs

    def tearDown(self):
        if hasattr(self, "_old_template_dirs"):
            settings.TEMPLATE_DIRS = self._old_template_dirs


class TestMessages(BaseTest):
    def test_messages(self):
        Message.new_message(
            self.brosner, [self.jtauber], "Really?", "You can't be serious")

        self.assertEqual(Thread.inbox(self.brosner).count(), 0)
        self.assertEqual(Thread.inbox(self.jtauber).count(), 1)

        thread = Thread.inbox(self.jtauber)[0]

        Message.new_reply(thread, self.jtauber, "Yes, I am.")

        self.assertEqual(Thread.inbox(self.brosner).count(), 1)
        self.assertEqual(Thread.inbox(self.jtauber).count(), 1)

        Message.new_reply(thread, self.brosner, "If you say so...")
        Message.new_reply(thread, self.jtauber, "Indeed I do")

        self.assertEqual(
            Thread.objects.get(pk=thread.pk).latest_message.content,
            "Indeed I do")
        self.assertEqual(
            Thread.objects.get(pk=thread.pk).first_message.content,
            "You can't be serious")

    def test_ordered(self):
        t1 = Message.new_message(
            self.brosner, [self.jtauber], "Subject",
            "A test message").thread
        t2 = Message.new_message(
            self.brosner, [self.jtauber], "Another",
            "Another message").thread
        t3 = Message.new_message(
            self.brosner, [self.jtauber], "Pwnt",
            "Haha I'm spamming your inbox").thread
        self.assertEqual(Thread.ordered([t2, t1, t3]), [t3, t2, t1])


class TestMessageViews(BaseTest):
    template_dirs = [
        os.path.join(os.path.dirname(__file__), "templates")
    ]

    def test_create_message(self):
        with self.login("brosner", "abc123"):
            response = self.get("messages_inbox")
            self.assertEqual(response.status_code, 200)

            response = self.get("message_create")
            self.assertEqual(response.status_code, 200)

            data = {
                "subject": "The internet is down.",
                "content": "Does this affect any of our sites?",
                "to_user": str(self.jtauber.id)
            }

            response = self.post("message_create", data=data)
            self.assertEqual(response.status_code, 302)

            self.assertEqual(Thread.inbox(self.jtauber).count(), 1)
            self.assertEqual(Thread.inbox(self.brosner).count(), 0)

            response = self.get("message_create", user_id=self.jtauber.id)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "selected=\"selected\">jtauber</option>")

            thread_id = Thread.inbox(self.jtauber).get().id

            response = self.get("messages_thread_detail", thread_id=thread_id)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "Does this affect any of our sites?")

        with self.login("jtauber", "abc123"):
            response = self.get("messages_inbox")
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, "Does this affect")

            response = self.get("messages_thread_detail", thread_id=thread_id)
            self.assertContains(response, "Does this affect")

            response = self.post("messages_thread_delete", thread_id=thread_id)
            self.assertEqual(response.status_code, 302)
            self.assertEqual(Thread.inbox(self.jtauber).count(), 0)

            data = {
                "content": "Nope, the internet being down doesn't affect us.",
            }

            response = self.post("messages_thread_detail", thread_id=thread_id, data=data)
            self.assertEqual(response.status_code, 302)
            self.assertEqual(Thread.inbox(self.brosner).count(), 1)
            self.assertEqual(
                Thread.inbox(self.brosner).get().messages.count(),
                2
            )
            self.assertEqual(Thread.unread(self.jtauber).count(), 0)

    def test_urls(self):
        self.assertEqual(reverse("message_create", args=[10]), "/create/10/")


class TestTemplateTags(BaseTest):
    def test_unread(self):
        thread = Message.new_message(self.brosner, [self.jtauber], "Why did you the internet?", "I demand to know.").thread
        tmpl = """{% load pinax_messages_tags %}{% if thread|unread:user %}UNREAD{% else %}READ{% endif %}"""
        self.assert_renders(
            tmpl,
            Context({"thread": thread, "user": self.jtauber}),
            "UNREAD"
        )
        self.assert_renders(
            tmpl,
            Context({"thread": thread, "user": self.brosner}),
            "READ",
        )
